------------------------------------------------------------------------------
--                                                                          --
--                         GNAT COMPILER COMPONENTS                         --
--                                                                          --
--                             E X P _ I N T R                              --
--                                                                          --
--                                 B o d y                                  --
--                                                                          --
--                            $Revision: 1.7 $                              --
--                                                                          --
--           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          --
--                                                                          --
-- GNAT is free software;  you can  redistribute it  and/or modify it under --
-- terms of the  GNU General Public License as published  by the Free Soft- --
-- ware  Foundation;  either version 2,  or (at your option) any later ver- --
-- sion.  GNAT is distributed in the hope that it will be useful, but WITH- --
-- OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY --
-- or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License --
-- for  more details.  You should have  received  a copy of the GNU General --
-- Public License  distributed with GNAT;  see file COPYING.  If not, write --
-- to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. --
--                                                                          --
------------------------------------------------------------------------------

with Atree;    use Atree;
with Einfo;    use Einfo;
with Exp_Util; use Exp_Util;
with Namet;    use Namet;
with Nmake;    use Nmake;
with Nlists;   use Nlists;
with Output;   use Output;
with Rtsfind;  use Rtsfind;
with Sem;      use Sem;
with Sem_Util; use Sem_Util;
with Sinfo;    use Sinfo;
with Snames;   use Snames;
with Tbuild;   use TBuild;

package body Exp_Intr is

   -----------------------
   -- Local Subprograms --
   -----------------------

   procedure Expand_Shift (N : Node_Id; E : Entity_Id; K : Node_Kind);
   --  Expand an intrinsic shift operation, N and E are from the call to
   --  Expand_Instrinsic_Call (call node and subprogram spec entity) and
   --  K is the kind for the shift node

   procedure Expand_Unc_Conversion (N : Node_Id; E : Entity_Id);
   --  Expand a call to an instantiation of Unchecked_Convertion into a node
   --  N_Unchecked_Type_Conversion.

   procedure Expand_Unc_Deallocation (N : Node_Id; E : Entity_Id);
   --  Expand a call to an instantiation of Unchecked_Deallocation into a node
   --  N_Free_Statement and appropriate context.


   ----------------------------
   -- Expand_Instrinsic_Call --
   ----------------------------

   procedure Expand_Intrinsic_Call (N : Node_Id; E : Entity_Id) is
      Nam : Name_Id;

   begin
      --  If the intrinsic subprogram is generic, gets its original name.

      if Present (Parent (E))
        and then Present (Generic_Parent (Parent (E)))
      then
         Nam := Chars (Generic_Parent (Parent (E)));
      else
         Nam := Chars (E);
      end if;

      if Nam = Name_Rotate_Left then
         Expand_Shift (N, E, N_Op_Rotate_Left);
      elsif Nam = Name_Rotate_Right then
         Expand_Shift (N, E, N_Op_Rotate_Right);
      elsif Nam = Name_Shift_Left then
         Expand_Shift (N, E, N_Op_Shift_Left);
      elsif Nam = Name_Shift_Right then
         Expand_Shift (N, E, N_Op_Shift_Right);
      elsif Nam = Name_Shift_Right_Arithmetic then
         Expand_Shift (N, E, N_Op_Shift_Right_Arithmetic);
      elsif Nam = Name_Unchecked_Conversion then
         Expand_Unc_Conversion (N, E);
      elsif Nam = Name_Unchecked_Deallocation then
         Expand_Unc_Deallocation (N, E);
      else
         pragma Assert (False); null;
      end if;
   end Expand_Intrinsic_Call;

   ------------------
   -- Expand_Shift --
   ------------------

   --  This procedure is used to convert a call to a shift function to the
   --  corresponding operator node. This conversion is not done by the usual
   --  circuit for converting calls to operator functions (e.g. "+"(1,2)) to
   --  operator nodes, because shifts are not predefined operators.

   --  As a result, whenever a shift is used in the source program, it will
   --  remain as a call until converted by this routine to the operator node
   --  form which Gigi is expecting to see.

   --  Note: it is possible for the expander to generate shift operator nodes
   --  directly, which will be analyzed in the normal manner by calling Analyze
   --  and Resolve. Such shift operator nodes will not be seen by Expand_Shift.

   procedure Expand_Shift (N : Node_Id; E : Entity_Id; K : Node_Kind) is
      Loc   : constant Source_Ptr := Sloc (N);
      Typ   : constant Entity_Id  := Etype (N);
      Left  : constant Node_Id    := First_Actual (N);
      Right : constant Node_Id    := Next_Actual (Left);
      Snode : Node_Id;

   begin
      Snode := New_Node (K, Loc);
      Set_Left_Opnd  (Snode, Relocate_Node (Left));
      Set_Right_Opnd (Snode, Relocate_Node (Right));
      Set_Chars      (Snode, Chars (E));
      Set_Etype      (Snode, Base_Type (Typ));
      Set_Entity     (Snode, E);

      --  Do the replacement. Note that we don't call Analyze and Resolve
      --  on this node, because it already got analyzed and resolved when
      --  it was a function call!

      Replace_Substitute_Tree (N, Snode);
      Set_Analyzed (N);

   end Expand_Shift;

   ---------------------------
   -- Expand_Unc_Conversion --
   ---------------------------

   procedure Expand_Unc_Conversion (N : Node_Id; E : Entity_Id) is
      Loc : constant Source_Ptr := Sloc (N);

   begin
      Replace_Substitute_Tree (N,
        Make_Unchecked_Type_Conversion (Loc,
          Subtype_Mark => New_Occurrence_Of (Etype (E), Loc),
          Expression   => Relocate_Node (First_Actual (N))));

      Set_Etype (N, Etype (E));
      Set_Analyzed (N);
   end Expand_Unc_Conversion;

   -----------------------------
   -- Expand_Unc_Deallocation --
   -----------------------------

   --  Generate the following Code :
   --    if Arg /= null then
   --       <if Has_Controlled (Designated_Type (Typ)) then>
   --          Deep_Finalize (Arg.all);
   --       <elsif Is_Controlled (Designated_Type (Typ)) then>
   --          Finalize (Arg.all);
   --       Free (Arg);
   --       Arg := Null;
   --    end if;

   procedure Expand_Unc_Deallocation (N : Node_Id; E : Entity_Id) is
      Loc        : constant Source_Ptr := Sloc (N);
      Arg        : constant Node_Id    := First_Actual (N);
      Typ        : constant Entity_Id  := Etype (Arg);
      Stmts      : constant List_Id    := New_List;
      Free_Node  : Node_Id;
      Code       : List_Id;
      Arg_Entity : Entity_Id;

   begin
      Prepare_Multi_Use_Of_Expr (Arg, Arg_Entity, Code);

      if Present (Code) then
         Insert_List_Before_And_Analyze (N, Code);
      end if;

      if Has_Controlled (Designated_Type (Typ)) then

         null;  --  Not Implemented Yet ???

      elsif Is_Controlled (Designated_Type (Typ)) then

         --  Generate the Finalization call

         Append_To (Stmts,
           Make_Procedure_Call_Statement (Loc,
             Name => New_Reference_To (RTE (RE_Finalize_One), Loc),
             Parameter_Associations => New_List (
               New_Reference_To (
                 Finalization_Chain_Entity (Scope (Typ)), Loc),

               Make_Unchecked_Type_Conversion (Loc,
                 Subtype_Mark =>
                   New_Reference_To (RTE (RE_Finalizable), Loc),
                 Expression => New_Ref_To_Expr (Arg_Entity, Loc)))));
      end if;

      Free_Node :=
        Make_Free_Statement (Loc,
          Expression => New_Ref_To_Expr (Arg_Entity, Loc));

      Set_Storage_Pool (Free_Node, Associated_Storage_Pool (Etype (Arg)));

      if Present (Storage_Pool (Free_Node)) then
         Set_Procedure_To_Call (Free_Node,
           Find_Prim_Op (Etype (Storage_Pool (Free_Node)), Name_Deallocate));
      end if;

      Append_To (Stmts, Free_Node);

      Append_To (Stmts,
        Make_Assignment_Statement (Loc,
          Name => New_Ref_To_Expr (Arg_Entity, Loc),
          Expression => Make_Null (Loc)));

      Replace_Substitute_Tree (N,
        Make_If_Statement (Loc,
          Condition =>
            Make_Op_Ne (Loc,
              Left_Opnd  => New_Ref_To_Expr (Arg_Entity, Loc),
              Right_Opnd => Make_Null (Loc)),

          Then_Statements => Stmts));

      Analyze (N);
   end Expand_Unc_Deallocation;

end Exp_Intr;


----------------------
-- REVISION HISTORY --
----------------------

--  ----------------------------
--  revision 1.5
--  date: Thu Jul  7 23:40:05 1994;  author: schonber
--  (Expand_Shift): the type of the node is the base type of its argument,
--   as for other arithmetic operations.
--  ----------------------------
--  revision 1.6
--  date: Thu Aug 18 16:26:13 1994;  author: comar
--  (Expand_Unc_Deallocation): Use Prepare_Multi_Use_Of_Expr/New_Ref_To_Expr
--   instead of Evaluate_Once which was not appropriate.
--  Fix bug reported by  bbard@@bbard.hac.com (Unchecked_Deallocation doesn't
--   null pointer) on July 25.
--  ----------------------------
--  revision 1.7
--  date: Thu Aug 18 20:09:09 1994;  author: dewar
--  Minor reformatting
--  ----------------------------
--  New changes after this line.  Each line starts with: "--  "
